searchData={"items":[{"type":"module","title":"LlmEx","doc":"LlmEx is a flexible client for interacting with various LLM providers.\n\nThis library provides a unified API for working with different LLM services,\nincluding local models via Ollama and cloud services like Anthropic.","ref":"LlmEx.html"},{"type":"module","title":"Getting Started - LlmEx","doc":"Add LlmEx to your dependencies in mix.exs:\n\n```elixir\ndef deps do\n  [\n    {:llm_ex, \"~> 0.1.0\"}\n  ]\nend\n```\n\nConfigure your provider API keys (if needed) in your config.exs:\n\n```elixir\nconfig :llm_ex,\n  anthropic_api_key: System.get_env(\"ANTHROPIC_API_KEY\")\n```","ref":"LlmEx.html#module-getting-started"},{"type":"module","title":"Usage - LlmEx","doc":"Create a client module and specify the provider:\n\n```elixir\ndefmodule MyApp.LlmClient do\n  use LlmEx.Client, provider: LlmEx.Providers.OllamaClient\n  \n  def get_knowledge do\n    with_tool(MyTool)\n    |> with_prompt(\"What do you know about Elixir?\")\n    |> stream()\n  end\nend\n```\n\nProcess the streaming response:\n\n```elixir\ndef handle_info({:llm_response, _id, content}, socket) do\n  # Handle the response content\n  {:noreply, socket}\nend\n```","ref":"LlmEx.html#module-usage"},{"type":"function","title":"LlmEx.client/2","doc":"Creates a new client with the specified provider.","ref":"LlmEx.html#client/2"},{"type":"function","title":"Parameters - LlmEx.client/2","doc":"* `provider` - The LLM provider module to use\n  * `opts` - Additional configuration options","ref":"LlmEx.html#client/2-parameters"},{"type":"function","title":"Examples - LlmEx.client/2","doc":"iex> client = LlmEx.client(LlmEx.Providers.OllamaClient)\n    iex> is_map(client)\n    true\n    iex> client.provider == LlmEx.Providers.OllamaClient\n    true","ref":"LlmEx.html#client/2-examples"},{"type":"function","title":"LlmEx.stream/1","doc":"Streams the response from the LLM provider.","ref":"LlmEx.html#stream/1"},{"type":"function","title":"Parameters - LlmEx.stream/1","doc":"* `client` - The client configuration","ref":"LlmEx.html#stream/1-parameters"},{"type":"function","title":"Examples - LlmEx.stream/1","doc":"iex> client = LlmEx.client(LlmEx.Providers.OllamaClient) |> LlmEx.with_prompt(\"Hello\")\n    iex> # Function will call provider.stream_chat which returns :ok\n    iex> # This is just a mock example for doctest\n    iex> client.provider != nil\n    true","ref":"LlmEx.html#stream/1-examples"},{"type":"function","title":"LlmEx.with_message/2","doc":"Adds a message to the conversation history.","ref":"LlmEx.html#with_message/2"},{"type":"function","title":"Parameters - LlmEx.with_message/2","doc":"* `client` - The client configuration\n  * `message` - The message to add","ref":"LlmEx.html#with_message/2-parameters"},{"type":"function","title":"Examples - LlmEx.with_message/2","doc":"iex> client = LlmEx.client(LlmEx.Providers.OllamaClient)\n    iex> message = %LlmEx.Types.Message{role: :user, content: \"Hello\"}\n    iex> result = LlmEx.with_message(client, message)\n    iex> length(result.messages)\n    1","ref":"LlmEx.html#with_message/2-examples"},{"type":"function","title":"LlmEx.with_prompt/2","doc":"Sets the prompt for the client.","ref":"LlmEx.html#with_prompt/2"},{"type":"function","title":"Parameters - LlmEx.with_prompt/2","doc":"* `client` - The client configuration\n  * `prompt` - The prompt text","ref":"LlmEx.html#with_prompt/2-parameters"},{"type":"function","title":"Examples - LlmEx.with_prompt/2","doc":"iex> client = LlmEx.client(LlmEx.Providers.OllamaClient)\n    iex> result = LlmEx.with_prompt(client, \"What is Elixir?\")\n    iex> result.prompt\n    \"What is Elixir?\"","ref":"LlmEx.html#with_prompt/2-examples"},{"type":"function","title":"LlmEx.with_system/2","doc":"Sets the system message for the client.","ref":"LlmEx.html#with_system/2"},{"type":"function","title":"Parameters - LlmEx.with_system/2","doc":"* `client` - The client configuration\n  * `system` - The system message text","ref":"LlmEx.html#with_system/2-parameters"},{"type":"function","title":"Examples - LlmEx.with_system/2","doc":"iex> client = LlmEx.client(LlmEx.Providers.OllamaClient)\n    iex> result = LlmEx.with_system(client, \"You are a helpful assistant.\")\n    iex> result.system\n    \"You are a helpful assistant.\"","ref":"LlmEx.html#with_system/2-examples"},{"type":"function","title":"LlmEx.with_tool/2","doc":"Adds a tool to the client configuration.","ref":"LlmEx.html#with_tool/2"},{"type":"function","title":"Parameters - LlmEx.with_tool/2","doc":"* `client` - The client configuration\n  * `tool` - The tool to add","ref":"LlmEx.html#with_tool/2-parameters"},{"type":"function","title":"Examples - LlmEx.with_tool/2","doc":"iex> client = LlmEx.client(LlmEx.Providers.OllamaClient)\n    iex> result = LlmEx.with_tool(client, :test_tool)\n    iex> result.tools\n    [:test_tool]","ref":"LlmEx.html#with_tool/2-examples"},{"type":"behaviour","title":"LlmEx.ClientBehaviour","doc":"Defines a common behaviour interface for all LLM clients.\nThis module establishes a contract that all LLM client implementations must fulfill.","ref":"LlmEx.ClientBehaviour.html"},{"type":"macro","title":"LlmEx.ClientBehaviour.__using__/1","doc":"Invoked when a module uses LlmEx.ClientBehaviour.\nImplements default functionality for the client.","ref":"LlmEx.ClientBehaviour.html#__using__/1"},{"type":"callback","title":"LlmEx.ClientBehaviour.message_to_provider_format/1","doc":"Converts a Message struct to the provider-specific format.","ref":"LlmEx.ClientBehaviour.html#c:message_to_provider_format/1"},{"type":"callback","title":"Parameters - LlmEx.ClientBehaviour.message_to_provider_format/1","doc":"* `message` - The Message struct to convert","ref":"LlmEx.ClientBehaviour.html#c:message_to_provider_format/1-parameters"},{"type":"callback","title":"LlmEx.ClientBehaviour.stream_chat/4","doc":"Streams a chat response from the LLM to the given process.","ref":"LlmEx.ClientBehaviour.html#c:stream_chat/4"},{"type":"callback","title":"Parameters - LlmEx.ClientBehaviour.stream_chat/4","doc":"* `messages` - List of Message structs representing the conversation history\n  * `message_id` - ID for the response message\n  * `pid` - The process ID to stream the response to\n  * `opts` - Options for the request (model, temperature, etc.)","ref":"LlmEx.ClientBehaviour.html#c:stream_chat/4-parameters"},{"type":"callback","title":"LlmEx.ClientBehaviour.tool_to_provider_format/1","doc":"Converts a Tool struct to the provider-specific format.","ref":"LlmEx.ClientBehaviour.html#c:tool_to_provider_format/1"},{"type":"callback","title":"Parameters - LlmEx.ClientBehaviour.tool_to_provider_format/1","doc":"* `tool` - The Tool struct to convert","ref":"LlmEx.ClientBehaviour.html#c:tool_to_provider_format/1-parameters"},{"type":"behaviour","title":"LlmEx.ProviderBehaviour","doc":"Defines a common behaviour interface for all LLM providers.\nThis module establishes a contract that all LLM provider implementations must fulfill.","ref":"LlmEx.ProviderBehaviour.html"},{"type":"macro","title":"LlmEx.ProviderBehaviour.__using__/1","doc":"Invoked when a module uses LlmEx.ProviderBehaviour.\nImplements default functionality for the provider.","ref":"LlmEx.ProviderBehaviour.html#__using__/1"},{"type":"callback","title":"LlmEx.ProviderBehaviour.message_to_provider_format/1","doc":"Converts a Message struct to the provider-specific format.","ref":"LlmEx.ProviderBehaviour.html#c:message_to_provider_format/1"},{"type":"callback","title":"Parameters - LlmEx.ProviderBehaviour.message_to_provider_format/1","doc":"* `message` - The Message struct to convert","ref":"LlmEx.ProviderBehaviour.html#c:message_to_provider_format/1-parameters"},{"type":"callback","title":"LlmEx.ProviderBehaviour.stream_chat/4","doc":"Streams a chat response from the LLM to the given process.","ref":"LlmEx.ProviderBehaviour.html#c:stream_chat/4"},{"type":"callback","title":"Parameters - LlmEx.ProviderBehaviour.stream_chat/4","doc":"* `messages` - List of Message structs representing the conversation history\n  * `message_id` - ID for the response message\n  * `pid` - The process ID to stream the response to\n  * `opts` - Options for the request (model, temperature, etc.)","ref":"LlmEx.ProviderBehaviour.html#c:stream_chat/4-parameters"},{"type":"callback","title":"LlmEx.ProviderBehaviour.tool_to_provider_format/1","doc":"Converts a Tool struct to the provider-specific format.","ref":"LlmEx.ProviderBehaviour.html#c:tool_to_provider_format/1"},{"type":"callback","title":"Parameters - LlmEx.ProviderBehaviour.tool_to_provider_format/1","doc":"* `tool` - The Tool struct to convert","ref":"LlmEx.ProviderBehaviour.html#c:tool_to_provider_format/1-parameters"},{"type":"module","title":"LlmEx.Providers.Anthropic","doc":"Client for the Anthropic Claude API","ref":"LlmEx.Providers.Anthropic.html"},{"type":"module","title":"LlmEx.Providers.OllamaClient","doc":"Client for communicating with a local LLM server (Ollama).\nProvides functionality to send prompts and stream responses back to a specified process.","ref":"LlmEx.Providers.OllamaClient.html"},{"type":"function","title":"LlmEx.Providers.OllamaClient.message_to_provider_format/1","doc":"Convert a Message struct to local LLM format","ref":"LlmEx.Providers.OllamaClient.html#message_to_provider_format/1"},{"type":"function","title":"LlmEx.Providers.OllamaClient.stream_chat/4","doc":"Streams a chat response from the local LLM server to the given pid.","ref":"LlmEx.Providers.OllamaClient.html#stream_chat/4"},{"type":"function","title":"Parameters - LlmEx.Providers.OllamaClient.stream_chat/4","doc":"* `messages` - List of Message structs representing the conversation history\n  * `message_id` - ID for the response message\n  * `pid` - The process ID to stream the response to\n  * `opts` - Options for the request\n    * `:model` - The model to use (default: llama3.2)\n    * `:temperature` - Temperature parameter for response randomness\n    * `:tools` - List of Tool structs available to the LLM","ref":"LlmEx.Providers.OllamaClient.html#stream_chat/4-parameters"},{"type":"function","title":"LlmEx.Providers.OllamaClient.tool_to_provider_format/1","doc":"Convert a Tool struct to local LLM format","ref":"LlmEx.Providers.OllamaClient.html#tool_to_provider_format/1"},{"type":"module","title":"LlmEx.StreamProcessor","doc":"Shared functionality for processing streaming responses from LLM providers.\nHandles common patterns for streaming responses and managing events.","ref":"LlmEx.StreamProcessor.html"},{"type":"function","title":"LlmEx.StreamProcessor.accumulate_json_delta/5","doc":"Process JSON deltas and accumulate them for tools that send partial JSON.","ref":"LlmEx.StreamProcessor.html#accumulate_json_delta/5"},{"type":"function","title":"Parameters - LlmEx.StreamProcessor.accumulate_json_delta/5","doc":"* `partial_json` - The partial JSON string to process\n  * `tool_key` - Unique key for the tool (usually {message_id, index})\n  * `get_fn` - Function to get the current state\n  * `put_fn` - Function to store the updated state\n  * `field` - Field to update (default: :input)","ref":"LlmEx.StreamProcessor.html#accumulate_json_delta/5-parameters"},{"type":"function","title":"Returns - LlmEx.StreamProcessor.accumulate_json_delta/5","doc":"* The updated state","ref":"LlmEx.StreamProcessor.html#accumulate_json_delta/5-returns"},{"type":"function","title":"LlmEx.StreamProcessor.handle_sse_stream/3","doc":"Handle line-by-line streaming with SSE (Server-Sent Events) format.","ref":"LlmEx.StreamProcessor.html#handle_sse_stream/3"},{"type":"function","title":"Parameters - LlmEx.StreamProcessor.handle_sse_stream/3","doc":"* `data` - The data chunk to process\n  * `state` - Current state including buffer and processing details\n  * `process_event_fn` - Function to process each event","ref":"LlmEx.StreamProcessor.html#handle_sse_stream/3-parameters"},{"type":"function","title":"LlmEx.StreamProcessor.parse_sse_line/1","doc":"Parse a single SSE (Server-Sent Events) line into event type and data.","ref":"LlmEx.StreamProcessor.html#parse_sse_line/1"},{"type":"function","title":"Parameters - LlmEx.StreamProcessor.parse_sse_line/1","doc":"* `line` - The SSE line to parse","ref":"LlmEx.StreamProcessor.html#parse_sse_line/1-parameters"},{"type":"function","title":"Returns - LlmEx.StreamProcessor.parse_sse_line/1","doc":"* `{:event, event_type, parsed_data}` - For event lines with JSON data\n  * `{:data, data}` - For data lines\n  * `:unknown` - For other lines","ref":"LlmEx.StreamProcessor.html#parse_sse_line/1-returns"},{"type":"function","title":"LlmEx.StreamProcessor.stream_processor/4","doc":"Process a streaming response with a custom handler function.","ref":"LlmEx.StreamProcessor.html#stream_processor/4"},{"type":"function","title":"Parameters - LlmEx.StreamProcessor.stream_processor/4","doc":"* `handler_func` - Function to process each data chunk\n  * `message_id` - ID of the message being processed\n  * `pid` - Process to send events to\n  * `opts` - Additional options","ref":"LlmEx.StreamProcessor.html#stream_processor/4-parameters"},{"type":"function","title":"Returns - LlmEx.StreamProcessor.stream_processor/4","doc":"* A function that can be used with Req.post! :into option","ref":"LlmEx.StreamProcessor.html#stream_processor/4-returns"},{"type":"module","title":"LlmEx.ToolHandler","doc":"Shared functionality for handling tools across different LLM clients.\nProvides common methods for executing tool calls, parsing arguments, and processing results.","ref":"LlmEx.ToolHandler.html"},{"type":"function","title":"LlmEx.ToolHandler.execute_tool_call/3","doc":"Execute a tool call and return the result.","ref":"LlmEx.ToolHandler.html#execute_tool_call/3"},{"type":"function","title":"Parameters - LlmEx.ToolHandler.execute_tool_call/3","doc":"* `tool_name` - Name of the tool to execute\n  * `args` - Arguments to pass to the tool\n  * `opts` - Additional options","ref":"LlmEx.ToolHandler.html#execute_tool_call/3-parameters"},{"type":"function","title":"Returns - LlmEx.ToolHandler.execute_tool_call/3","doc":"* Map containing either a result or an error","ref":"LlmEx.ToolHandler.html#execute_tool_call/3-returns"},{"type":"function","title":"LlmEx.ToolHandler.parse_tool_arguments/1","doc":"Parse tool arguments from various formats into a map.","ref":"LlmEx.ToolHandler.html#parse_tool_arguments/1"},{"type":"function","title":"Parameters - LlmEx.ToolHandler.parse_tool_arguments/1","doc":"* `args` - Arguments to parse (string, map, or other)","ref":"LlmEx.ToolHandler.html#parse_tool_arguments/1-parameters"},{"type":"function","title":"Returns - LlmEx.ToolHandler.parse_tool_arguments/1","doc":"* Map of parsed arguments","ref":"LlmEx.ToolHandler.html#parse_tool_arguments/1-returns"},{"type":"function","title":"LlmEx.ToolHandler.process_tool_result/1","doc":"Process various tool result formats into a standardized format.","ref":"LlmEx.ToolHandler.html#process_tool_result/1"},{"type":"function","title":"Returns - LlmEx.ToolHandler.process_tool_result/1","doc":"* Map with either a \"result\" or \"error\" key","ref":"LlmEx.ToolHandler.html#process_tool_result/1-returns"},{"type":"module","title":"LlmEx.Tools.Declarative","doc":"Provides a declarative API for defining tool schemas.\n\nThis module implements macros similar to Phoenix.Component's attr macro,\nallowing tool developers to define their tools in a more declarative style.","ref":"LlmEx.Tools.Declarative.html"},{"type":"macro","title":"LlmEx.Tools.Declarative.param/3","doc":"Declares a parameter for a tool.","ref":"LlmEx.Tools.Declarative.html#param/3"},{"type":"macro","title":"Arguments - LlmEx.Tools.Declarative.param/3","doc":"* `name` - an atom or string defining the name of the parameter\n* `type` - an atom defining the type of the parameter (\":string\", \":integer\", etc.)\n* `opts` - a keyword list of options. Defaults to `[]`","ref":"LlmEx.Tools.Declarative.html#param/3-arguments"},{"type":"macro","title":"Options - LlmEx.Tools.Declarative.param/3","doc":"* `:required` - marks a parameter as required. Defaults to `false`\n* `:description` - documentation for the parameter\n* `:enum` - a list of allowed values for the parameter\n* `:default` - the default value for the parameter if not provided","ref":"LlmEx.Tools.Declarative.html#param/3-options"},{"type":"macro","title":"Examples - LlmEx.Tools.Declarative.param/3","doc":"param :action, :string, required: true, enum: [\"increment\", \"decrement\", \"reset\"],\n                           description: \"The action to perform\"","ref":"LlmEx.Tools.Declarative.html#param/3-examples"},{"type":"macro","title":"LlmEx.Tools.Declarative.tool_description/1","doc":"Sets the description of the tool.","ref":"LlmEx.Tools.Declarative.html#tool_description/1"},{"type":"macro","title":"Examples - LlmEx.Tools.Declarative.tool_description/1","doc":"tool_description \"A stateful counter that increments, decrements, or resets\"","ref":"LlmEx.Tools.Declarative.html#tool_description/1-examples"},{"type":"macro","title":"LlmEx.Tools.Declarative.tool_name/1","doc":"Sets the name of the tool.","ref":"LlmEx.Tools.Declarative.html#tool_name/1"},{"type":"macro","title":"Examples - LlmEx.Tools.Declarative.tool_name/1","doc":"tool_name \"counter\"","ref":"LlmEx.Tools.Declarative.html#tool_name/1-examples"},{"type":"behaviour","title":"LlmEx.Tools.ToolBehaviour","doc":"Behaviour for LlmEx tools.\n\nThis module defines the behaviour for all tools in the LlmEx system.\nTools can be stateful or stateless, and must implement the required callbacks.","ref":"LlmEx.Tools.ToolBehaviour.html"},{"type":"behaviour","title":"Declarative Tool Definition - LlmEx.Tools.ToolBehaviour","doc":"Tools can be defined using a declarative API similar to Phoenix.Component's attr macro:\n\n```elixir\ndefmodule MyTool do\n  use LlmEx.Tools.ToolBehaviour\n\n  tool_name \"my_tool\"\n  tool_description \"A simple tool that does something useful\"\n\n  param :action, :string, required: true,\n                         enum: [\"start\", \"stop\", \"pause\"],\n                         description: \"The action to perform\"\n  param :amount, :integer, description: \"The amount to process\"\n\n  # ... rest of implementation\nend\n```","ref":"LlmEx.Tools.ToolBehaviour.html#module-declarative-tool-definition"},{"type":"behaviour","title":"State Management - LlmEx.Tools.ToolBehaviour","doc":"Tools can maintain state between calls. The state is passed to the `handle_call/2`\ncallback and returned as part of the result. For example:\n\n```elixir\ndef handle_call(arguments, state) do\n  # ... process arguments\n  new_state = %{state | count: state.count + 1}\n  {:ok, result, new_state}\nend\n```","ref":"LlmEx.Tools.ToolBehaviour.html#module-state-management"},{"type":"callback","title":"LlmEx.Tools.ToolBehaviour.handle_call/2","doc":"Handles a call to the tool with the given arguments and current state.\n\nThis function is called when a client requests the tool to be executed.\nIt should process the arguments and return `{:ok, result, new_state}` where\n`result` is a map that will be returned to the client and `new_state` is the\nupdated state of the tool.","ref":"LlmEx.Tools.ToolBehaviour.html#c:handle_call/2"},{"type":"callback","title":"Examples - LlmEx.Tools.ToolBehaviour.handle_call/2","doc":"def handle_call(arguments, state) do\n      # Create a result object with text content\n      result = %{\n        \"content\" => [%{\"type\" => \"text\", \"text\" => \"Example response\"}],\n        \"isError\" => false\n      }\n      {:ok, result, state}\n    end","ref":"LlmEx.Tools.ToolBehaviour.html#c:handle_call/2-examples"},{"type":"callback","title":"LlmEx.Tools.ToolBehaviour.init/1","doc":"Initializes the tool's state.\n\nThis function is called when the tool is first registered with the system.\nIt should return `{:ok, state}` where `state` can be any term that represents\nthe tool's initial state.","ref":"LlmEx.Tools.ToolBehaviour.html#c:init/1"},{"type":"callback","title":"Examples - LlmEx.Tools.ToolBehaviour.init/1","doc":"def init(_args) do\n      {:ok, %{count: 0}}\n    end","ref":"LlmEx.Tools.ToolBehaviour.html#c:init/1-examples"},{"type":"callback","title":"LlmEx.Tools.ToolBehaviour.tool_definition/0","doc":"Returns the tool definition for the tools/list endpoint.\n\nThis function should return a map with the tool's name, description, and input schema.","ref":"LlmEx.Tools.ToolBehaviour.html#c:tool_definition/0"},{"type":"callback","title":"Examples - LlmEx.Tools.ToolBehaviour.tool_definition/0","doc":"def tool_definition do\n      %{\n        \"name\" => \"echo\",\n        \"description\" => \"Echoes back the input arguments\",\n        \"inputSchema\" => %{\n          \"type\" => \"object\",\n          \"properties\" => %{\n            \"message\" => %{\n              \"type\" => \"string\",\n              \"description\" => \"The message to echo\"\n            }\n          },\n          \"required\" => [\"message\"]\n        }\n      }\n    end","ref":"LlmEx.Tools.ToolBehaviour.html#c:tool_definition/0-examples"},{"type":"module","title":"LlmEx.Types","doc":"Defines standardized types for LLM interactions across different providers.","ref":"LlmEx.Types.html"},{"type":"module","title":"LlmEx.Types.Config","doc":"Configuration for an LLM provider","ref":"LlmEx.Types.Config.html"},{"type":"type","title":"LlmEx.Types.Config.t/0","doc":"","ref":"LlmEx.Types.Config.html#t:t/0"},{"type":"module","title":"LlmEx.Types.Events","doc":"Event structs for LLM interactions","ref":"LlmEx.Types.Events.html"},{"type":"module","title":"LlmEx.Types.Events.Done","doc":"Event signaling the end of an LLM response","ref":"LlmEx.Types.Events.Done.html"},{"type":"type","title":"LlmEx.Types.Events.Done.t/0","doc":"","ref":"LlmEx.Types.Events.Done.html#t:t/0"},{"type":"module","title":"LlmEx.Types.Events.Response","doc":"Event for a text response from the LLM","ref":"LlmEx.Types.Events.Response.html"},{"type":"type","title":"LlmEx.Types.Events.Response.t/0","doc":"","ref":"LlmEx.Types.Events.Response.html#t:t/0"},{"type":"module","title":"LlmEx.Types.Events.ToolCallEvent","doc":"Event for a tool call from the LLM","ref":"LlmEx.Types.Events.ToolCallEvent.html"},{"type":"type","title":"LlmEx.Types.Events.ToolCallEvent.t/0","doc":"","ref":"LlmEx.Types.Events.ToolCallEvent.html#t:t/0"},{"type":"module","title":"LlmEx.Types.Message","doc":"Represents a message in the LLM conversation","ref":"LlmEx.Types.Message.html"},{"type":"type","title":"LlmEx.Types.Message.t/0","doc":"","ref":"LlmEx.Types.Message.html#t:t/0"},{"type":"module","title":"LlmEx.Types.Tool","doc":"Represents a tool definition for the LLM","ref":"LlmEx.Types.Tool.html"},{"type":"type","title":"LlmEx.Types.Tool.t/0","doc":"","ref":"LlmEx.Types.Tool.html#t:t/0"},{"type":"module","title":"LlmEx.Types.ToolCall","doc":"Represents a tool call request from the LLM","ref":"LlmEx.Types.ToolCall.html"},{"type":"type","title":"LlmEx.Types.ToolCall.t/0","doc":"","ref":"LlmEx.Types.ToolCall.html#t:t/0"},{"type":"behaviour","title":"VibeCluster.Llm.BaseLLMClient","doc":"Defines a common behaviour interface for all LLM clients.\nThis module establishes a contract that all LLM client implementations must fulfill.","ref":"VibeCluster.Llm.BaseLLMClient.html"},{"type":"function","title":"VibeCluster.Llm.BaseLLMClient.make_llm_request/4","doc":"Helper function to make HTTP requests to LLM providers with proper error handling","ref":"VibeCluster.Llm.BaseLLMClient.html#make_llm_request/4"},{"type":"function","title":"VibeCluster.Llm.BaseLLMClient.make_streaming_request/5","doc":"Helper function to make streaming HTTP requests to LLM providers with proper error handling","ref":"VibeCluster.Llm.BaseLLMClient.html#make_streaming_request/5"},{"type":"callback","title":"VibeCluster.Llm.BaseLLMClient.message_to_provider_format/1","doc":"Converts a Message struct to the provider-specific format.","ref":"VibeCluster.Llm.BaseLLMClient.html#c:message_to_provider_format/1"},{"type":"callback","title":"Parameters - VibeCluster.Llm.BaseLLMClient.message_to_provider_format/1","doc":"* `message` - The Message struct to convert","ref":"VibeCluster.Llm.BaseLLMClient.html#c:message_to_provider_format/1-parameters"},{"type":"callback","title":"VibeCluster.Llm.BaseLLMClient.stream_chat/4","doc":"Streams a chat response from the LLM to the given process.","ref":"VibeCluster.Llm.BaseLLMClient.html#c:stream_chat/4"},{"type":"callback","title":"Parameters - VibeCluster.Llm.BaseLLMClient.stream_chat/4","doc":"* `messages` - List of Message structs representing the conversation history\n  * `message_id` - ID for the response message\n  * `pid` - The process ID to stream the response to\n  * `opts` - Options for the request (model, temperature, etc.)","ref":"VibeCluster.Llm.BaseLLMClient.html#c:stream_chat/4-parameters"},{"type":"callback","title":"VibeCluster.Llm.BaseLLMClient.tool_to_provider_format/1","doc":"Converts a Tool struct to the provider-specific format.","ref":"VibeCluster.Llm.BaseLLMClient.html#c:tool_to_provider_format/1"},{"type":"callback","title":"Parameters - VibeCluster.Llm.BaseLLMClient.tool_to_provider_format/1","doc":"* `tool` - The Tool struct to convert","ref":"VibeCluster.Llm.BaseLLMClient.html#c:tool_to_provider_format/1-parameters"},{"type":"extras","title":"README","doc":"# 🧠 LlmEx\n\n[![Hex.pm](https://img.shields.io/hexpm/v/llm_ex.svg)](https://hex.pm/packages/llm_ex)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/llm_ex)\n[![License](https://img.shields.io/hexpm/l/llm_ex.svg)](https://opensource.org/licenses/MIT)\n\n**A beautiful, modular, and powerful Elixir client for LLMs**","ref":"readme.html"},{"type":"extras","title":"BETA WORK IN PROGRESS DO NOT USE IN PRODUCTION, RAPIDLY CHANGING - README","doc":"[Installation](#installation) •\n[Key Features](#key-features) •\n[Getting Started](#getting-started) •\n[API Reference](#api-reference) •\n[Examples](#examples) •\n[Advanced Usage](#advanced-usage) •\n[Contributing](#contributing) •\n[License](#license)","ref":"readme.html#beta-work-in-progress-do-not-use-in-production-rapidly-changing"},{"type":"extras","title":"💫 Overview - README","doc":"LlmEx is a powerful, expressive, and modular Elixir client for Large Language Models. It provides a clean, consistent interface to interact with various LLM providers while embracing Elixir's functional programming paradigm and the BEAM ecosystem's strengths.\n\nBuilt by the community, for the community - LlmEx aims to be the definitive open source solution for LLM integration in Elixir applications.\n\nWith LlmEx, you can:\n- 🚀 Interact with multiple LLM providers using a consistent interface\n- 🔧 Define and use tools with LLMs (function calling) through a declarative API\n- 📺 Stream responses for real-time interactions\n- 🧩 Create composable, pipeable client configurations\n- ⚡ Handle Server-Sent Events (SSE) with ease\n- 🔄 Integrate seamlessly with Phoenix LiveView, GenServers, and more","ref":"readme.html#overview"},{"type":"extras","title":"📦 Installation - README","doc":"Add `llm_ex` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:llm_ex, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"🌟 Key Features - README","doc":"- **Provider-agnostic interface**: Use the same code with different LLM providers\n- **Elegant client builder API**: Build clients using a clean, fluent interface\n- **First-class tool support**: Define tools that LLMs can use to perform actions\n- **Declarative tool definitions**: Express tool schemas in a clear, concise way\n- **Streaming support**: Stream responses in real-time for better UX\n- **Modular design**: Small, focused modules that are easy to test and maintain\n- **Pure Elixir**: No external dependencies beyond HTTP clients","ref":"readme.html#key-features"},{"type":"extras","title":"🚀 Getting Started - README","doc":"","ref":"readme.html#getting-started"},{"type":"extras","title":"Creating Your Client Module - README","doc":"LlmEx uses a simple behavior-based approach. Create your own client in seconds:\n\n```elixir\ndefmodule MyApp.LlmClient do\n  use LlmEx.ClientBehaviour\n  \n  # Specify which provider to use (can be overridden in options)\n  @provider LlmEx.Providers.OllamaClient\n  \n  @doc \"\"\"\n  Get a simple response from the LLM.\n  \"\"\"\n  def get_response(prompt) do\n    with_system(\"You are a helpful assistant.\")\n    |> with_user_message(prompt)\n    |> chat()\n  end\n  \n  @doc \"\"\"\n  Get a streamed response sent to the calling process.\n  \"\"\"\n  def get_streamed_response(prompt) do\n    with_system(\"You are a helpful assistant.\")\n    |> with_user_message(prompt)\n    |> stream_chat(self())\n    \n    # Handle the streamed response in the calling process\n    listen_for_response()\n  end\n  \n  defp listen_for_response do\n    receive do\n      {:llm_content, content} ->\n        IO.write(content)\n        listen_for_response()\n      {:llm_done, _} ->\n        IO.puts(\"\\nResponse complete\")\n    after\n      30_000 -> IO.puts(\"Timeout\")\n    end\n  end\n  \n  @doc \"\"\"\n  Use a calculator tool to solve math problems.\n  \"\"\"\n  def solve_math_problem(problem) do\n    with_system(\"You are a math assistant that can solve problems.\")\n    |> with_tool(Calculator)\n    |> with_user_message(problem)\n    |> chat()\n  end\nend\n```\n\nThen use it in your application:\n\n```elixir\n# Simple response\nMyApp.LlmClient.get_response(\"What is Elixir?\")\n\n# Streamed response\nMyApp.LlmClient.get_streamed_response(\"Tell me a story about a time-traveling programmer.\")\n\n# With tool usage\nMyApp.LlmClient.solve_math_problem(\"What is 25 * 16?\")\n```","ref":"readme.html#creating-your-client-module"},{"type":"extras","title":"Defining Tools - README","doc":"LlmEx offers two ways to define tools that LLMs can use: a standard approach and a declarative approach.\n\n#### Standard Approach with ToolBehaviour\n\nUse the `LlmEx.Tools.ToolBehaviour` module to implement tools:\n\n```elixir\ndefmodule MyApp.Tools.Calculator do\n  @behaviour LlmEx.Tools.ToolBehaviour\n  \n  # Implement the required callback\n  @impl LlmEx.Tools.ToolBehaviour\n  def tool_definition do\n    %{\n      \"name\" => \"calculator\",\n      \"description\" => \"A calculator that can perform basic operations.\",\n      \"inputSchema\" => %{\n        \"type\" => \"object\",\n        \"properties\" => %{\n          \"operation\" => %{\n            \"type\" => \"string\",\n            \"enum\" => [\"add\", \"subtract\", \"multiply\", \"divide\"]\n          },\n          \"a\" => %{\"type\" => \"number\"},\n          \"b\" => %{\"type\" => \"number\"}\n        },\n        \"required\" => [\"operation\", \"a\", \"b\"]\n      }\n    }\n  end\n  \n  # Initialize the tool (for stateful tools)\n  @impl LlmEx.Tools.ToolBehaviour\n  def init(_opts) do\n    {:ok, %{}}\n  end\n  \n  # Handle tool calls\n  @impl LlmEx.Tools.ToolBehaviour\n  def handle_call(params, state) do\n    a = params[\"a\"]\n    b = params[\"b\"]\n    \n    result = case params[\"operation\"] do\n      \"add\" -> a + b\n      \"subtract\" -> a - b\n      \"multiply\" -> a * b\n      \"divide\" when b != 0 -> a / b\n      \"divide\" -> \"Cannot divide by zero\"\n      _ -> \"Invalid operation\"\n    end\n    \n    response = %{\n      \"content\" => [%{\"type\" => \"text\", \"text\" => \"The result is: #{result}\"}],\n      \"isError\" => false\n    }\n    \n    {:ok, response, state}\n  end\nend\n```\n\n#### Declarative Approach\n\nThe declarative approach provides a more concise and expressive way to define tools:\n\n```elixir\ndefmodule MyApp.Tools.DeclarativeCalculator do\n  use LlmEx.Tools.ToolBehaviour\n  \n  # Define the tool metadata\n  tool_name \"calculator\"\n  tool_description \"A calculator that can perform basic operations.\"\n  \n  # Define parameters\n  param :operation, :string, \n    description: \"The operation to perform\", \n    enum: [\"add\", \"subtract\", \"multiply\", \"divide\"],\n    required: true\n    \n  param :a, :number, \n    description: \"First number\", \n    required: true\n    \n  param :b, :number, \n    description: \"Second number\", \n    required: true\n  \n  # Initialize the tool (required by the behaviour)\n  @impl LlmEx.Tools.ToolBehaviour\n  def init(_opts) do\n    {:ok, %{}} # Initialize with empty state\n  end\n  \n  # Handle tool calls\n  @impl LlmEx.Tools.ToolBehaviour\n  def handle_call(params, state) do\n    a = params[\"a\"]\n    b = params[\"b\"]\n    \n    result = case params[\"operation\"] do\n      \"add\" -> a + b\n      \"subtract\" -> a - b\n      \"multiply\" -> a * b\n      \"divide\" when b != 0 -> a / b\n      \"divide\" -> \"Cannot divide by zero\"\n      _ -> \"Invalid operation\"\n    end\n    \n    response = %{\n      \"content\" => [%{\"type\" => \"text\", \"text\" => \"The result is: #{result}\"}],\n      \"isError\" => false\n    }\n    \n    {:ok, response, state}\n  end\nend\n```","ref":"readme.html#defining-tools"},{"type":"extras","title":"📚 API Reference - README","doc":"","ref":"readme.html#api-reference"},{"type":"extras","title":"Using the ClientBehaviour - README","doc":"The `LlmEx.ClientBehaviour` module provides these functions:\n\n- `with_provider(state, provider)` - Set the LLM provider\n- `with_system(state, system_message)` - Set the system message\n- `with_user_message(state, content)` - Add a user message\n- `with_assistant_message(state, content)` - Add an assistant message\n- `with_tool(state, tool)` - Add a tool to the client\n- `with_prompt(state, prompt)` - Set a single prompt\n- `chat(state, opts \\\\ [])` - Send a chat request and wait for the response\n- `stream_chat(state, pid, opts \\\\ [])` - Stream a chat response to the given process","ref":"readme.html#using-the-clientbehaviour"},{"type":"extras","title":"Defining Tools - README","doc":"Tools must implement the `LlmEx.Tools.ToolBehaviour` which requires:\n\n- `tool_definition/0` - Returns the tool definition (name, description, input schema)\n- `init/1` - Initializes the tool state\n- `handle_call/2` - Handles a tool call with parameters and state","ref":"readme.html#defining-tools"},{"type":"extras","title":"🔍 Supported Providers - README","doc":"- **Ollama**: Local models through the Ollama API\n- **Anthropic**: Claude models through Anthropic's API\n- **More Coming Soon**: OpenAI, Mistral, and many more!","ref":"readme.html#supported-providers"},{"type":"extras","title":"🤝 Contributing - README","doc":"Contributions are welcome and appreciated! LlmEx is an open source project that aims to provide the best possible LLM experience for Elixir developers.\n\n1. Fork the repository\n2. Create your feature branch (`git checkout -b feature/amazing-feature`)\n3. Commit your changes (`git commit -m 'Add some amazing feature'`)\n4. Push to the branch (`git push origin feature/amazing-feature`)\n5. Open a Pull Request\n\nPlease make sure your code follows the Elixir style guide and includes tests.","ref":"readme.html#contributing"},{"type":"extras","title":"📜 License - README","doc":"Distributed under the MIT License. See `LICENSE` for more information.","ref":"readme.html#license"},{"type":"extras","title":"🙏 Acknowledgements - README","doc":"- The Elixir community for its supportive and collaborative spirit\n- All contributors who have helped make this project better\n- José Valim and the Elixir core team for creating such an amazing language\n\n---\n\n \n\n📚 **Documentation**: [hexdocs.pm/llm_ex](https://hexdocs.pm/llm_ex)  \n\nBuilt with ❤️ by the Elixir community","ref":"readme.html#acknowledgements"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}